#include "stdafx.h"
#include <memory.h>
#include "DataBrokerConnectWrapper.h"

#define MAX_CONN 1  // Max number of connections


// Function     : CDataBrokerConnectWrapper::CDataBrokerConnectWrapper
// Description  : Constructor
// Parameter    : 
//              | [in] hAvSyncWrapper  @AvSyncWrapper handle
//              | [in] pfStatus        @Status callback function
//              | [in] pfAV            @AV callback function
//              | [in] dwStatusContext @Status context
//              | [in] dwAVContext     @AV context
// Return value : N/A
//
CDataBrokerConnectWrapper::CDataBrokerConnectWrapper(TDataBrokerStatusCallback pfStatus, TDataBrokerAVCallback pfAV, DWORD dwStatusContext, DWORD dwAVContext)
{
	memset(&m_tParam, 0, sizeof(m_tParam));

	m_tParam.pfStatus  = pfStatus;
	m_tParam.pfAV      = pfAV;
	
	m_tParam.dwStatusContext  = dwStatusContext;
	m_tParam.dwAVContext      = dwAVContext;
	
	InitialModule();  // Initialize DataBroker and create connection instance
}

// Function     : CDataBrokerConnectWrapper::~CDataBrokerConnectWrapper
// Description  : Destructor
// Parameter    : N/A
// Return value : N/A
//
CDataBrokerConnectWrapper::~CDataBrokerConnectWrapper()
{
	ReleaseModule();  // Delete connection instance and release DataBroker
}

// Function     : CDataBrokerConnectWrapper::InitialModule
// Description  : Initialize DataBroker and create connection instance
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CDataBrokerConnectWrapper::InitialModule()
{
	SCODE scRet;

	// STEP 1: Initialize DataBroker
	DWORD dwMediaCodecType = mctJPEG | mctH263 | mctMP4V | mctG7221 | mctG729A | mctAAC | mctGAMR | mctSAMR;

	scRet = DataBroker_Initial(&m_hDatabroker, MAX_CONN, m_tParam.pfStatus, m_tParam.pfAV, dwMediaCodecType, 0, DATABROKER_VERSION);

	if	(IS_FAIL(scRet))
	{
		sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "DataBroker_Initial fail! (ERROR: %X)", scRet);

		MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

		return S_FAIL;
	}

	// STEP 2: Create DataBroker connection instance
	if	(m_hDatabroker != NULL)
	{
		scRet = CreateConnection();

		if	(IS_FAIL(scRet))
		{
			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CDataBrokerConnectWrapper::ReleaseModule
// Description  : Delete connection instance and release DataBroker
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CDataBrokerConnectWrapper::ReleaseModule()
{
	SCODE scRet;

	if	(m_hDatabroker != NULL)
	{
		// STEP 1: Delete connection instance
		if	(m_hConn != NULL)
		{
			scRet = DeleteConnection();

			if	(IS_FAIL(scRet))
			{
				return S_FAIL;
			}
		}

		// STEP 2: Release DataBroker
		scRet = DataBroker_Release(&m_hDatabroker);

		if	(IS_FAIL(scRet))
		{
			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CDataBrokerConnectWrapper::CreateConnection
// Description  : Create DataBroker connection instance
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CDataBrokerConnectWrapper::CreateConnection()
{
	SCODE scRet;

	scRet = DataBroker_CreateConnection(m_hDatabroker, &m_hConn);  // Create connection instance

	if	(IS_FAIL(scRet))
	{
		sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "DataBroker_CreateConnection fail! (ERROR: %X)", scRet);

		MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

		return S_FAIL;
	}

	return S_OK;
}

// Function     : CDataBrokerConnectWrapper::DeleteConnection
// Description  : Delete connection instance
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CDataBrokerConnectWrapper::DeleteConnection()
{
	SCODE scRet;

	scRet = DataBroker_DeleteConnection(m_hDatabroker, &m_hConn);  // Delete connection instance

	if	(IS_FAIL(scRet))
	{
		sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "DataBroker_DeleteConnection fail! (ERROR: %X)", scRet);

		MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

		return S_FAIL;
	}

	return S_OK;
}

// Function     : CDataBrokerConnectWrapper::Connect
// Description  : Connect to remote server
// Parameter    : 
//              | [in] wHttpPort     @HTTP port
//              | [in] pszIPAddr     @IP address
//              | [in] pszServerType @Server type
//              | [in] pszUserID     @User ID
//              | [in] pszPassword   @Password
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CDataBrokerConnectWrapper::Connect(WORD wHttpPort, TCHAR *pszIPAddr, TCHAR *pszServerType, TCHAR *pszUserID, TCHAR *pszPassword)
{
	m_tParam.wHttpPort = wHttpPort;

	memcpy_s(m_tParam.szIPAddr, DB_PARAM_BUF_SIZE, pszIPAddr, DB_PARAM_BUF_SIZE);
	memcpy_s(m_tParam.szServerType, DB_PARAM_BUF_SIZE, pszServerType, DB_PARAM_BUF_SIZE);
	memcpy_s(m_tParam.szUserID, DB_PARAM_BUF_SIZE, pszUserID, DB_PARAM_BUF_SIZE);
	memcpy_s(m_tParam.szPassword, DB_PARAM_BUF_SIZE, pszPassword, DB_PARAM_BUF_SIZE);

	SCODE scRet;

	if	(m_hConn != NULL)
	{	
		// STEP 1: Set channel options
		scRet = SetConnectionOptions();

		if	(IS_FAIL(scRet))
		{
			return S_FAIL;
		}

		// STEP 2: Connect to remote server
		scRet = DataBroker_Connect(m_hConn);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "DataBroker_Connect fail! (ERROR: %X)", scRet);

			MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

			return S_FAIL;
		}

		// When connect to remote server successfully, DataBroker callback with status type: eOnConnectionInfo
	}

	return S_OK;
}

// Function     : CDataBrokerConnectWrapper::Disconnect
// Description  : Disconnect from remote server
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CDataBrokerConnectWrapper::Disconnect()
{
	SCODE scRet;

	if	(m_hConn != NULL)
	{	
		// Disconnect from remote server
		scRet = DataBroker_Disconnect(m_hConn);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "DataBroker_Disconnect fail! (ERROR: %X)", scRet);

			MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

			return S_FAIL;
		}
	}

	// When disconnect from remote server successfully, DataBroker callback with status type: eOnStopped

	return S_OK;
}


// Function     : CDataBrokerConnectWrapper::SetConnectionOptions
// Description  : Set connection option
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CDataBrokerConnectWrapper::SetConnectionOptions()
{
	SCODE scRet;

	TDataBrokerConnectionOptions tDBConnOptions;

	memset(&tDBConnOptions, 0, sizeof(tDBConnOptions));

	tDBConnOptions.wHttpPort    = m_tParam.wHttpPort;
	tDBConnOptions.pzIPAddr     = m_tParam.szIPAddr;
	tDBConnOptions.pzServerType = m_tParam.szServerType;
	tDBConnOptions.pzUID        = m_tParam.szUserID;
	tDBConnOptions.pzPWD        = m_tParam.szPassword;

	tDBConnOptions.dwFlags = (eConOptProtocolAndMediaType | eConOptHttpPort | (eConOptStatusContext | eConOptStatusCallback) | (eConOptAVContext | eConOptAVContext));

	tDBConnOptions.pfStatus        = m_tParam.pfStatus;
	tDBConnOptions.pfAV            = m_tParam.pfAV;	
	tDBConnOptions.dwStatusContext = m_tParam.dwStatusContext;
	tDBConnOptions.dwAVContext     = m_tParam.dwAVContext;	
	tDBConnOptions.dwProtocolType  = eptTCP;
	tDBConnOptions.dwMediaType     = (emtVideo | emtAudio);

	scRet = DataBroker_SetConnectionOptions(m_hConn, &tDBConnOptions);

	if	(IS_FAIL(scRet))
	{
		sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "DataBroker_SetConnectionOptions fail! (ERROR: %X)\n", scRet);

		OutputDebugString(m_szErrMsgBuf);

		return S_FAIL;
	}
	
	return S_OK;
}
