#include "stdafx.h"
#include <memory.h>
#include "AvSyncLiveSnapshotWrapper.h"

#define AV_PATHBUF_SIZE MAX_PATH  // Max size of save snapshot file path buffer

// Function     : CAvSyncLiveSnapshotWrapper::CAvSyncLiveSnapshotWrapper
// Description  : Constructor
// Parameter    : 
//              | [in] hDlg             @Dialog window handle
//              | [in] hDisplay         @Display window handle
//              | [in] hBMP             @Bitmap handle
//              | [in] pfStatus         @Status callback function
//              | [in] pfDisplay        @Display callback function
//              | [in] dwStatusContext  @Status context
//              | [in] dwDisplayContext @Display context
// Return value : N/A
//
CAvSyncLiveSnapshotWrapper::CAvSyncLiveSnapshotWrapper(HWND hDlg, HWND hDisplay, HBITMAP hBMP, LPSTATUSCALLBACK pfStatus, LPDISPLAYCALLBACK pfDisplay, DWORD dwStatusContext, DWORD dwDisplayContext)
{
	m_hAvSync      = NULL;
	m_hLiveChannel = NULL;

	m_dwSnapshotFileCount = 0;
	
	memset(&m_tParam, 0, sizeof(m_tParam));

	m_tParam.hDlg     = hDlg;
	m_tParam.hDisplay = hDisplay;
	m_tParam.hBMP     = hBMP;

	m_tParam.pfStatus  = pfStatus;
	m_tParam.pfDisplay = pfDisplay;
	
	m_tParam.dwStatusContext  = dwStatusContext;
	m_tParam.dwDisplayContext = dwDisplayContext;

	InitialModule();  // Initialize AvSynchronizer and create live channel
}

// Function     : CAvSyncLiveSnapshotWrapper::~CAvSyncLiveSnapshotWrapper
// Description  : Destructor
// Parameter    : N/A
// Return value : N/A
//
CAvSyncLiveSnapshotWrapper::~CAvSyncLiveSnapshotWrapper()
{	
	ReleaseModule();  // Delete live channel and release AvSynchronizer
}

// Function     : CAvSyncLiveSnapshotWrapper::InitialModule
// Description  : Initialize AvSynchronizer and create live channel
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncLiveSnapshotWrapper::InitialModule()
{	
	SCODE scRet;

	// STEP 1: Initialize AvSynchronizer
	scRet = AvSynchronizer_InitialEx(&m_hAvSync, m_tParam.pfStatus, m_tParam.pfDisplay, m_tParam.hDlg, AUDIOOUT_FOCUS_NORMAL, 0, AVSYNCHRONIZER_VERSION, 0, RGB(0, 0, 0));

	if	(IS_FAIL(scRet))
	{
		sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_InitialEx fail! (ERROR: %X)", scRet);

		MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

		return S_FAIL;
	}

	// STEP 2: Create live channel
	if	(m_hAvSync != NULL)
	{
		scRet = CreateChannel();

		if	(IS_FAIL(scRet))
		{
			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CAvSyncLiveSnapshotWrapper::ReleaseModule
// Description  : Delete live channel and release AvSynchronizer
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncLiveSnapshotWrapper::ReleaseModule()
{
	SCODE scRet;
	
	if	(m_hAvSync != NULL)
	{
		// STEP 1: Delete live channel
		if	(m_hLiveChannel != NULL)
		{
			scRet = DeleteChannel();

			if	(IS_FAIL(scRet))
			{
				return S_FAIL;
			}
		}

		// STEP 2: Release AvSynchronizer
		scRet = AvSynchronizer_Release(&m_hAvSync);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_Release fail! (ERROR: %X)", scRet);

			MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CAvSyncLiveSnapshotWrapper::CreateChannel
// Description  : Create live channel
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncLiveSnapshotWrapper::CreateChannel()
{	
	// STEP 1: Set channel options
	TCHANNELOPTION tCHOption;

	memset(&tCHOption, 0, sizeof(tCHOption));

	tCHOption.dwFlags = CH_STATUSCB | CH_DISPLAYCB | CH_STATUSCONTEXT | CH_DISPLAYCONTEXT | CH_ALLBORDER | CH_BITMAP;

	tCHOption.pfStatus  = m_tParam.pfStatus; 
	tCHOption.pfDisplay = m_tParam.pfDisplay;

	tCHOption.dwStatusContext = m_tParam.dwStatusContext;
	tCHOption.dwDisplayContext = m_tParam.dwDisplayContext;
	
	tCHOption.dwTopBorderSize    = 20;
	tCHOption.dwLeftBorderSize   = 0;
	tCHOption.dwBottomBorderSize = 0;
	tCHOption.dwRightBorderSize  = 0;

	tCHOption.hBMP     = m_tParam.hBMP;
	tCHOption.hDisplay = m_tParam.hDisplay;
	
	SCODE scRet;

	// STEP 2: Create live channel
	scRet = AvSynchronizer_CreateChannel(m_hAvSync, &m_hLiveChannel, tCHOption);

	if	(IS_FAIL(scRet))
	{
		sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_CreateChannel fail! (ERROR: %X)", scRet);

		MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

		return S_FAIL;
	}
	
	return S_OK;
}

// Function     : CAvSyncLiveSnapshotWrapper::DeleteChannel
// Description  : Delete live channel
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncLiveSnapshotWrapper::DeleteChannel()
{
	SCODE scRet;

	// Delete live channel 
	scRet = AvSynchronizer_DeleteChannel(m_hAvSync, &m_hLiveChannel);

	if	(IS_FAIL(scRet))
	{
		sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_DeleteChannel fail! (ERROR: %X)", scRet);

		MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

		return S_FAIL;
	}

	return S_OK;
}

// Function     : CAvSyncLiveSnapshotWrapper::Start
// Description  : Start live channel
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncLiveSnapshotWrapper::Start()
{	
	SCODE scRet;

	// Start live channel
	if	(m_hLiveChannel != NULL)
	{
		scRet = AvSynchronizer_StartChannel(m_hLiveChannel, DRAW_CONNECTING);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_StartChannel fail! (ERROR: %X)", scRet);

			MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CAvSyncLiveSnapshotWrapper::Stop
// Description  : Stop live channel
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncLiveSnapshotWrapper::Stop()
{
	SCODE scRet;
	
	// Stop live channel
	if	(m_hLiveChannel != NULL)
	{
		scRet = AvSynchronizer_StopChannel(m_hLiveChannel);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_StopChannel fail! (ERROR: %X)", scRet);

			MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CAvSyncLiveSnapshotWrapper::UpdateChannelMediaType
// Description  : Update channel media type
// Parameter    :
//              | [in] dwMediaType @Media type
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncLiveSnapshotWrapper::UpdateChannelMediaType(DWORD dwMediaType)
{
	SCODE scRet;

	if	(m_hLiveChannel != NULL)
	{	
		TUPDATECHANNELOPTION tUpdataCHOption;

		tUpdataCHOption.dwFlags    = UPCH_MEDIATYPE;
		tUpdataCHOption.tMediaType = EMEDIATYPE(dwMediaType);
	
		scRet = AvSynchronizer_UpdateChannelSettings(m_hLiveChannel, tUpdataCHOption);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_UpdateChannelSettings fail! (ERROR: %X)\n", scRet);

			OutputDebugString(m_szErrMsgBuf);

			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CAvSyncLiveSnapshotWrapper::UpdateChannelShowBMP
// Description  : Update channel settings to show bitmap
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncLiveSnapshotWrapper::UpdateChannelShowBMP()
{
	SCODE scRet;

	if	(m_hLiveChannel != NULL)
	{	
		TUPDATECHANNELOPTION tUpdataCHOption;

		tUpdataCHOption.dwFlags = UPCH_SHOW_BMP;
	
		scRet = AvSynchronizer_UpdateChannelSettings(m_hLiveChannel, tUpdataCHOption);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_UpdateChannelSettings fail! (ERROR: %X)\n", scRet);

			OutputDebugString(m_szErrMsgBuf);

			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CAvSyncLiveSnapshotWrapper::InputEncodedMediaFrame
// Description  : Input the encoded media frame into channel
// Parameter    :
//              | [in] pMediaDataPacket @Input media data packet info
// Return value :
//              | S_OK                        @Success
//              | S_FAIL                      @FAIL
//              | AVSYNCHRONIZER_E_QUEUE_FULL @Queue full
//
SCODE CAvSyncLiveSnapshotWrapper::InputEncodedMediaFrame(TMediaDataPacketInfo *pMediaDataPacket)
{
	SCODE scRet;

	if	(m_hLiveChannel != NULL)
	{		
		scRet = AvSynchronizer_InputEncodedMediaFrame(m_hLiveChannel, pMediaDataPacket);

		if (scRet != S_OK)
		{
			// The return value "AVSYNCHRONIZER_E_QUEUE_FULL" is used to notify the caller that the queue is full.
			// When used in DataBrokerAVCallback function, if the callback function recieves "AVSYNCHRONIZER_E_QUEUE_FULL",
			// it returns "DATABROKER_S_FRAME_NOT_HANDLED" back to DataBroker. Then DataBroker stop putting media data into
			// the queue until it recieves "S_OK."
			//
			if	(scRet == AVSYNCHRONIZER_E_QUEUE_FULL)
			{
				sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer queue full!\n", scRet);

				OutputDebugString(m_szErrMsgBuf);

				return scRet;
			}
			else
			{
				sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_InputEncodedMediaFrame fail! (ERROR: %X)\n", scRet);

				OutputDebugString(m_szErrMsgBuf);

				return S_FAIL;
			}
		}
	}

	return S_OK;
}

// Function     : CAvSyncLiveSnapshotWrapper::Snapshot
// Description  : Perform snapshot and save as JPEG file
// Parameter    : 
//              | [in] pszSaveDirectoryPath @Save snapshot file directory path
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncLiveSnapshotWrapper::Snapshot(char *pszSaveDirectoryPath)
{
	SCODE scRet;

	TSNAPSHOT tSnapshot;
	
	// STEP 1: Perfrom snapshot and store the info in "tSnapshot" structure
	
	// If caller wants to get snapshot with JPEG format, the specified flag could be set as ((PF_JPEG << 16) | SPECIFY_FMT)
	//
	scRet = AvSynchronizer_GetCurrentSnapShot(m_hLiveChannel, &tSnapshot, ((PF_JPEG << 16) | SPECIFY_FMT) | ORIGINAL_SIZE);

	if	(IS_FAIL(scRet))
	{
		sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_GetCurrentSnapShot fail! (ERROR: %X)", scRet);

		OutputDebugString(m_szErrMsgBuf);

		return S_FAIL;
	}

	// STEP 2: Open a file and write the data at buffer "pDataStart" with size "dwdataSize"
	FILE *pSnapshotFile;
	
	char szFilePathBuf[AV_PATHBUF_SIZE] = "\0";

	sprintf_s(szFilePathBuf, AV_PATHBUF_SIZE, "%s\\Snapshot-[%d].jpg", pszSaveDirectoryPath, m_dwSnapshotFileCount);

	fopen_s(&pSnapshotFile, szFilePathBuf, "wb");

	if	(pSnapshotFile != NULL)
	{
		fwrite(tSnapshot.pDataStart, sizeof(BYTE), tSnapshot.dwdataSize, pSnapshotFile);

		fclose(pSnapshotFile);
	}

	// STEP 3: Free the snapshot created by AvSynchronizer_GetCurrentSnapShot
	scRet = AvSynchronizer_FreePicture(&tSnapshot);

	if	(IS_FAIL(scRet))
	{
		sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_FreePicture fail! (ERROR: %X)", scRet);

		OutputDebugString(m_szErrMsgBuf);

		return S_FAIL;
	}

	m_dwSnapshotFileCount++;  // Increase the file count

	return S_OK;
}